@@ -489,6 +489,9 @@ function test_typed_array()
     a = new Uint16Array(buffer, 2);
     a[0] = -1;
 
+    a = new Float16Array(buffer, 8, 1);
+    a[0] = 1;
+
     a = new Float32Array(buffer, 8, 1);
     a[0] = 1;
 
@@ -593,7 +596,7 @@ function test_json()
  ]
 ]`);
 
-    assert_json_error('\n"  @\\x"');
+    assert_json_error('\n"  \\@x"');
     assert_json_error('\n{ "a": @x }"');
 }
 
@@ -748,6 +751,51 @@ function test_regexp()
     assert(a, ["123a23", "3"]);
     a = /()*?a/.exec(",");
     assert(a, null);
+
+    /* test \b escape */
+    assert(/[\q{a\b}]/.test("a\b"), true);
+    assert(/[\b]/.test("\b"), true);
+    
+    /* test case insensitive matching (test262 hardly tests it) */
+    assert("aAbBcC#4".replace(/\p{Lower}/gu,"X"), "XAXBXC#4");
+
+    assert("aAbBcC#4".replace(/\p{Lower}/gui,"X"), "XXXXXX#4");
+    assert("aAbBcC#4".replace(/\p{Upper}/gui,"X"), "XXXXXX#4");
+    assert("aAbBcC#4".replace(/\P{Lower}/gui,"X"), "XXXXXXXX");
+    assert("aAbBcC#4".replace(/\P{Upper}/gui,"X"), "XXXXXXXX");
+    assert("aAbBcC".replace(/[^b]/gui, "X"), "XXbBXX");
+    assert("aAbBcC".replace(/[^A-B]/gui, "X"), "aAbBXX");
+
+    assert("aAbBcC#4".replace(/\p{Lower}/gvi,"X"), "XXXXXX#4");
+    assert("aAbBcC#4".replace(/\P{Lower}/gvi,"X"), "aAbBcCXX");
+    assert("aAbBcC#4".replace(/[^\P{Lower}]/gvi,"X"), "XXXXXX#4");
+    assert("aAbBcC#4".replace(/\P{Upper}/gvi,"X"), "aAbBcCXX");
+    assert("aAbBcC".replace(/[^b]/gvi, "X"), "XXbBXX");
+    assert("aAbBcC".replace(/[^A-B]/gvi, "X"), "aAbBXX");
+    assert("aAbBcC".replace(/[[a-c]&&B]/gvi, "X"), "aAXXcC");
+    assert("aAbBcC".replace(/[[a-c]--B]/gvi, "X"), "XXbBXX");
+    
+    assert("abcAbC".replace(/[\q{AbC}]/gvi,"X"), "XX");
+    /* Note: SpiderMonkey and v8 may not be correct */
+    assert("abcAbC".replace(/[\q{BC|A}]/gvi,"X"), "XXXX");
+    assert("abcAbC".replace(/[\q{BC|A}--a]/gvi,"X"), "aXAX");
+
+    /* case where lastIndex points to the second element of a
+       surrogate pair */
+    a = /(?:)/gu;
+    a.lastIndex = 1;
+    a.exec("🐱");
+    assert(a.lastIndex, 0);
+
+    a.lastIndex = 1;
+    a.exec("a\udc00");
+    assert(a.lastIndex, 1);
+
+    a = /\u{10000}/vgd;
+    a.lastIndex = 1;
+    a = a.exec("\u{10000}_\u{10000}");
+    assert(a.indices[0][0], 0);
+    assert(a.indices[0][1], 2);
 }
 
 function test_symbol()
